////////////////////////////////////////////////////////////////////////////////
//
//
//   Author       : Bogdan Popescu
//   Organization : University of Cincinnati
//   Program      : gofield2
//   Version      : 2.014
//   Date         : Dec 10, 2014
//   Tested       : g++ 4.6.3 x64 (Ubuntu 12.04 LTS x64)
//                  g++ 4.8.2 x64 (Ubuntu 14.04 LTS x64)     
//
//
//
////////////////////////////////////////////////////////////////////////////////
//
//    MASSCLEAN (MASSive CLuster Evolution and ANalysis Package)
//    Copyright (C) 2007-2014 BOGDAN POPESCU and MARGARET HANSON
//
//    gofield2.cc is part of MASSCLEAN.
//
//    MASSCLEAN is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation.
//
//    MASSCLEAN is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with MASSCLEAN; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//    or go to : http://www.gnu.org/licenses/gpl.html
//
////////////////////////////////////////////////////////////////////////////////


#include <cstdlib>
#include <ctime>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <math.h>
#include <ctype.h> 
#include <iomanip> 

#include <sys/time.h>  
#include <time.h>
#include <unistd.h>


#define NOCOLUMNS 1
#define PI 3.14159265


using namespace std;


int alllines(char *name)
{
//////////////////////
////
//// 
////

  int NOLINES;

  char oneline[400];
  NOLINES=0;

  ifstream data0;

  data0.open(name);

  if (data0.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;
  }

  while (!data0.eof())
  {
     if (!isdigit(data0.peek()) || !isspace(data0.peek()))
	{
	    NOLINES=NOLINES+1;

            data0.getline(oneline,400,'\n');

	    if(data0.eof()) break;
	}

     if (isdigit(data0.peek()) || isspace(data0.peek()))
       {
            NOLINES=NOLINES+1;

            data0.getline(oneline,400,'\n');

            if(data0.eof()) break;
        }

   if(data0.eof()) break;
   }

  data0.close();

  return NOLINES;

}


int alldatalines(char *name, int NOLINES)
{
//////////////////////
////
//// 
////

  int i,k;

  int NODATALINES;

  double read_column;

  NODATALINES=0;

  ifstream data1;

  data1.open(name);

  if (data1.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;
  }

for (k=0;k<(NOLINES);k++)
 {
   if (isdigit(data1.peek()) || isspace(data1.peek()))
       {
       for (i=0;i<(NOCOLUMNS);i++)  
            {                      
            data1 >> read_column ;
            
            }
        NODATALINES=NODATALINES+1;

        if(data1.eof()) break;
        }

   if (!isdigit(data1.peek()) || !isspace(data1.peek()))
	  {data1.ignore(400,'\n');data1.peek(); }

 if(data1.eof()) break;
 }

  data1.close();

return NODATALINES;

}


int alldatacolumns(char *name)
{
//////////////////////
////
//// 
////

  int NOCOL;

  double col;

  NOCOL=0;

  ifstream data3;

  data3.open(name);

  if (data3.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;
  }

  while (data3.peek()=='#') {data3.ignore(400,'\n'); }

  while (1)
  {
  data3 >> col ;

  NOCOL=NOCOL+1;

  if(data3.peek()=='\n') break;

  if(data3.eof()) break;
  }

  data3.close();

 return NOCOL;

}




int datalinescolumn(char *name, int i)

{
 if (i==1) return alldatalines(name,alllines(name));
 if (i==2) return alldatacolumns(name);
}



///////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
///////////
///////////  MAIN PROGRAM
///////////
//////////////////////////////////////////////////////////////////////////////////



int main()
{

  cout << "\n";


  cout <<"MASSCLEAN 2.014 (MASSive CLuster Evolution and ANalysis Package)"<<"\n";

  cout <<"Copyright (C) 2007-2014 BOGDAN POPESCU and MARGARET HANSON,"<<"\n";

  cout <<"under GNU General Public License. For details see 'COPYING'."<<"\n";

  cout <<"The program 'goFIELD2' is part of MASSCLEAN 2.014 CC (Clever Crocodile)."<<"\n";

  cout << "\n";
  cout <<"http://massclean.org/"<< '\n';
  cout <<"http://bogdan.massclean.org/"<< '\n';

  cout << "\n";


  cout <<"Popescu & Hanson 2009, AJ, 138, 1724"<< '\n';
  cout <<"Popescu & Hanson 2010, ApJL, 713, L21"<< '\n';
  cout <<"Popescu & Hanson 2010, ApJ, 724, 296"<< '\n';
  cout <<"Popescu, Hanson & Elmegreen 2012, ApJ, 751, 122"<< '\n';
  cout <<"Popescu & Hanson 2014, ApJ, 780, 27"<< '\n';


  cout << "\n";
  cout << "\n";


  /// prototypes
  int alllines(char*);
  int alldatalines(char*,int);
  int alldatacolumns(char*);
  int datalinescolumn(char*,int);

//  srand((unsigned)time(0));
////////////////////////////////////////////////////////
timeval timv;
gettimeofday(&timv, NULL);

int randomini;
randomini=timv.tv_usec;
srand(randomini);
///////////////////////////////////////////////////////



chdir("ini.files");

///////////////

cout << "Reading file cluster.ini...";

int STARS;

int Xmax,XmaxIR;
int Ymax,YmaxIR;
double MinMag;
double MaxMag;


double X,Y,MAG;
int VERBOSE;

double mass,dd,FOV;

ifstream clusterfile;
clusterfile.open("cluster.ini");

  if (clusterfile.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;
  }

  while (clusterfile.peek()=='#') { clusterfile.ignore(300,'\n');}

clusterfile >> VERBOSE; clusterfile.ignore(300,'\n');
clusterfile >> mass; clusterfile.ignore(300,'\n');
clusterfile >> dd; clusterfile.ignore(300,'\n');
clusterfile >> FOV; clusterfile.ignore(300,'\n');
clusterfile >> XmaxIR; clusterfile.ignore(300,'\n');
clusterfile >> YmaxIR; clusterfile.ignore(300,'\n');
clusterfile >> Xmax; clusterfile.ignore(300,'\n');
clusterfile >> Ymax; clusterfile.ignore(300,'\n');


clusterfile.ignore(300,'\n');

clusterfile.close();


// distance modulus


  double apparent;

  apparent=5.*log10(dd/10.);

cout << "...done !"<<'\n';
cout << "\n";

/////////////////

chdir("..");




  ///////////////
  
  double A_U,A_B,A_V,A_R,A_I,A_J,A_H,A_K;
  
  chdir("data"); 

cout << "Reading file ccm.filters...";

ifstream filterfile1;
filterfile1.open("ccm.filters");

  if (filterfile1.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;
  }

  while (filterfile1.peek()=='#') { filterfile1.ignore(300,'\n');}

filterfile1 >> A_U; filterfile1.ignore(300,'\n');
filterfile1 >> A_B; filterfile1.ignore(300,'\n');
filterfile1 >> A_V; filterfile1.ignore(300,'\n');
filterfile1 >> A_R; filterfile1.ignore(300,'\n');
filterfile1 >> A_I; filterfile1.ignore(300,'\n');
filterfile1 >> A_J; filterfile1.ignore(300,'\n');
filterfile1 >> A_H; filterfile1.ignore(300,'\n');
filterfile1 >> A_K; filterfile1.ignore(300,'\n');

filterfile1.ignore(300,'\n');

filterfile1.close();

cout << "...done !"<<'\n';


chdir(".."); 

/////////////////




int NOTREKBINS, NOCOLS;



/// dir 

chdir("isochrones");

chdir("grid");


NOTREKBINS=datalinescolumn("grid.dat",1);
NOCOLS=19;


//// grid file
//// 

ifstream opentrek;

opentrek.open("grid.dat");

  if (opentrek.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;
  }

  while (opentrek.peek()=='#') { opentrek.ignore(300,'\n');}


//chdir("..");

chdir(".."); 
chdir(".."); /// main directory

chdir("ini.files");

  int headonoff;  


  ifstream data13("header.ini");

  if (data13.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;
  }

  while (data13.peek()=='#') { data13.ignore(300,'\n');}

  data13 >> headonoff; 
  data13.ignore(300,'\n');

  data13.close();




cout << "Reading file field.ini...";

ifstream fieldfile;
fieldfile.open("field.ini");

  if (fieldfile.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;
  }

  while (fieldfile.peek()=='#') { fieldfile.ignore(300,'\n');}



char band;
double MAXLIMIT, MINLIMIT;
double STEP;
double SLOPE;
int NTOTALREF;
double A_U2,A_B2,A_V2,A_R2,A_I2,A_J2,A_H2,A_K2;
int extonoff;


fieldfile >> band;  fieldfile.ignore(300,'\n');
fieldfile >> SLOPE;  fieldfile.ignore(300,'\n');
fieldfile >> NTOTALREF;  fieldfile.ignore(300,'\n');
fieldfile >> MINLIMIT;  fieldfile.ignore(300,'\n');
fieldfile >> MAXLIMIT;  fieldfile.ignore(300,'\n');
fieldfile >> STEP;  fieldfile.ignore(300,'\n');
fieldfile >> extonoff;  fieldfile.ignore(300,'\n');
fieldfile >> A_U2;  fieldfile.ignore(300,'\n');
fieldfile >> A_B2;  fieldfile.ignore(300,'\n');
fieldfile >> A_V2;  fieldfile.ignore(300,'\n');
fieldfile >> A_R2;  fieldfile.ignore(300,'\n');
fieldfile >> A_I2;  fieldfile.ignore(300,'\n');
fieldfile >> A_J2;  fieldfile.ignore(300,'\n');
fieldfile >> A_H2;  fieldfile.ignore(300,'\n');
fieldfile >> A_K2;  fieldfile.ignore(300,'\n');

fieldfile.ignore(300,'\n');

cout << "...done !"<<'\n';
cout << "\n";

if (extonoff==0)
{
A_U=A_U2;
A_B=A_B2;
A_V=A_V2;
A_R=A_R2;
A_I=A_I2;
A_J=A_J2;
A_H=A_H2;
A_K=A_K2;
}




cout << "Computing...";


chdir(".."); /// main directory


chdir("data");


////  interpolation file
//// 
 
 ofstream interpfile("field.interp");

  if (interpfile.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;

  }
  
if (headonoff==1)
{  

  interpfile <<"##  "<< '\n';
  interpfile <<"##  MASSCLEAN 2.014 Clever Crocodile"<< '\n';
  interpfile <<"##  B.Popescu and M.M.Hanson, 2007-2014 "<< '\n';
  interpfile <<"##  http://massclean.org/"<< '\n';

  interpfile <<"##  "<< '\n';

  interpfile <<"##  Popescu & Hanson 2009, AJ, 138, 1724"<< '\n';
  interpfile <<"##  Popescu & Hanson 2010, ApJL, 713, L21"<< '\n';
  interpfile <<"##  Popescu & Hanson 2010, ApJ, 724, 296"<< '\n';
  interpfile <<"##  Popescu, Hanson & Elmegreen 2012, ApJ, 751, 122"<< '\n';
  interpfile <<"##  Popescu & Hanson 2014, ApJ, 780, 27"<< '\n';

  interpfile <<"##  "<< '\n';


  interpfile <<"##  field.interp  "<<'\n';
  interpfile <<"##   "<<'\n';
  interpfile <<"##     U           B          U-B          V          B-V          R          V-R          I          R-I          J          I-J          H          J-H         K           H-K    "<<'\n';
  interpfile <<"##     1           2           3           4           5           6           7           8           9          10          11          12          13         14           15     "<<'\n';
  interpfile <<"##   "<<'\n';
  
}

//// citire + scriere


interpfile.setf(ios::fixed,ios::floatfield);  
interpfile.precision(6);

double trekvalues[NOTREKBINS][NOCOLS];
int citlin, citcol;

int interpindex[4];

interpindex[0]=0;
interpindex[3]=NOTREKBINS-1;

int interpstep;
interpstep=int(NOTREKBINS/3.);

interpindex[1]= 1+(rand())%(int((interpstep-1)));
interpindex[2]= interpstep + 1+(rand())%(int((interpstep-1)));




int altindex;



for (citlin=0;citlin<(NOTREKBINS);citlin++)
 {
 
       {
       for (citcol=0;citcol<(NOCOLS);citcol++)  
            {                      
            opentrek >> trekvalues[citlin][citcol] ;
            
            }
      

for (altindex=0;altindex<4;altindex++)
{

if (citlin==interpindex[altindex])
{
interpfile << setw(12) <<trekvalues[citlin][7]+trekvalues[citlin][8]+trekvalues[citlin][9] + apparent + A_U<< setw(12) << trekvalues[citlin][7]+trekvalues[citlin][9]+ apparent + A_B<< setw(12) << trekvalues[citlin][8]<< setw(12) << trekvalues[citlin][7]+ apparent + A_V<< setw(12) << trekvalues[citlin][9]<< setw(12) << trekvalues[citlin][7] - trekvalues[citlin][10]+ apparent + A_R << setw(12) << trekvalues[citlin][10] << setw(12) << trekvalues[citlin][7] - trekvalues[citlin][11] + apparent + A_I<<  setw(12) << trekvalues[citlin][13]<< setw(12) << trekvalues[citlin][7] + trekvalues[citlin][18] - trekvalues[citlin][14] - trekvalues[citlin][11] + apparent + A_J << setw(12) <<  trekvalues[citlin][7] - trekvalues[citlin][11] - (trekvalues[citlin][7] + trekvalues[citlin][18] - trekvalues[citlin][14] - trekvalues[citlin][11])<< setw(12) << trekvalues[citlin][7] + trekvalues[citlin][18] - trekvalues[citlin][14] - trekvalues[citlin][11] - trekvalues[citlin][15]+ apparent + A_H<< setw(12) << trekvalues[citlin][15]<< setw(12) << trekvalues[citlin][7] - trekvalues[citlin][11] - trekvalues[citlin][14]+ apparent + A_K << setw(12) << trekvalues[citlin][16] << "\n";
}
}
        if(opentrek.eof()) break;
        }


 if(opentrek.eof()) break;
 }

  




interpfile.close();
opentrek.close();

chdir("..");
  

  
  
  chdir("data");
  
  ofstream file1("background.U.list");

  if (file1.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;

  }
  
  ofstream file2("background.B.list");

  if (file2.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;

  }
  
  ofstream file3("background.V.list");

  if (file3.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;

  }
  
  ofstream file4("background.R.list");

  if (file4.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;

  }
  
   
  ofstream file5("background.I.list");

  if (file5.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;

  }
  
   ofstream file6("background.J.list");

  if (file6.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;

  }
  
  ofstream file7("background.H.list");

  if (file7.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;

  }
  
  ofstream file8("background.K.list");

  if (file8.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;

  }
  
  ofstream file9("field.plot");

  if (file9.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;

  }
  
  

  file9 <<"##  "<< '\n';
  file9 <<"##  MASSCLEAN 2.014 Clever Crocodile"<< '\n';
  file9 <<"##  B.Popescu and M.M.Hanson, 2007-2014 "<< '\n';
  file9 <<"##  http://massclean.org/"<< '\n';

  file9 <<"##  "<< '\n';

  file9 <<"##  Popescu & Hanson 2009, AJ, 138, 1724"<< '\n';
  file9 <<"##  Popescu & Hanson 2010, ApJL, 713, L21"<< '\n';
  file9 <<"##  Popescu & Hanson 2010, ApJ, 724, 296"<< '\n';
  file9 <<"##  Popescu, Hanson & Elmegreen 2012, ApJ, 751, 122"<< '\n';
  file9 <<"##  Popescu & Hanson 2014, ApJ, 780, 27"<< '\n';

  file9 <<"##  "<< '\n';
  
  file9 <<"##  field.plot  "<<'\n';
  file9 <<"##   "<<'\n';
  file9 <<"##        U             B             V             R             I            J              H             K            x             y           "<<'\n';      

  file9 <<"##        1             2             3             4             5            6              7             8            9            10           "<<'\n';    

  
  
  chdir(".."); /// main directory
  



chdir("data");

fieldfile.close();

int juju, jojo;
double values[4][15], values2[4][15];

ifstream interpfield;
interpfield.open("field.interp");

  if (interpfield.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;
  }

  while (interpfield.peek()=='#') { interpfield.ignore(300,'\n');}


for (juju=0;juju<4;juju++)
 {
 
       {
       for (jojo=0;jojo<15;jojo++)  
            {                     
            interpfield >> values2[juju][jojo] ;
            values[juju][jojo]=values2[juju][jojo];
           
           
            }


if(interpfield.eof()) break;
        }

  

 if(interpfield.eof()) break;
 }

interpfield.close();
chdir("..");

double AREA; 
AREA=FOV*FOV/60./60.;
double BB;
BB=log10(NTOTALREF/AREA)-SLOPE*MINLIMIT;



double MAXLIMIT0;


MAXLIMIT0=MAXLIMIT;

MAXLIMIT=MINLIMIT-(log10(NTOTALREF/AREA)/SLOPE)- fmod(MINLIMIT-(log10(NTOTALREF/AREA)/SLOPE),1.);



int NOSTEPS; 
NOSTEPS=(int)((MINLIMIT-MAXLIMIT)/STEP-fmod((MINLIMIT-MAXLIMIT)/STEP,1.));



double DeltaMagU,DeltaMagB,DeltaMagV,DeltaMagR,DeltaMagI,DeltaMagJ,DeltaMagH,DeltaMagK;
double UB,BV,VR,RI,IJ,JH,HK;

int z,q;


int rest;

double ramas=0.;

int INTSTARS=0;

//// multiple steps loop

for (q=0;q<NOSTEPS;q++)
{

MaxMag=MAXLIMIT+STEP*q;
MinMag=MAXLIMIT+STEP*(q+1);
                                                             
if (fmod((pow(10.,SLOPE*MinMag+BB) - pow(10.,SLOPE*MaxMag+BB))*AREA,1.)<=0.5)
{ rest = 0; }

if (fmod((pow(10.,SLOPE*MinMag+BB) - pow(10.,SLOPE*MaxMag+BB))*AREA,1.)>0.5)
{ rest = 1; }

STARS= int((pow(10.,SLOPE*MinMag+BB)-pow(10.,SLOPE*MaxMag+BB))*AREA-fmod((pow(10.,SLOPE*MinMag+BB) - pow(10.,SLOPE*MaxMag+BB))*AREA,1.)+rest);




if (STARS==0)
{
ramas=ramas+(pow(10.,SLOPE*MinMag+BB)-pow(10.,SLOPE*MaxMag+BB))*AREA;

STARS=int((pow(10.,SLOPE*MinMag+BB)-pow(10.,SLOPE*MaxMag+BB))*AREA-fmod((pow(10.,SLOPE*MinMag+BB) - pow(10.,SLOPE*MaxMag+BB))*AREA,1.)+rest+ramas);

}

INTSTARS=INTSTARS+STARS;


for (z=0;z<STARS;z++)
{



X=rand()/(RAND_MAX+1.0);
Y=rand()/(RAND_MAX+1.0);

MAG=MinMag+rand()/(RAND_MAX+1.0)*(MaxMag-MinMag);

double selectie;
selectie=rand()/(RAND_MAX+1.0);

int selectie3;
selectie3 = (rand())%10;

int selectie2;
if (selectie3 >= 1)
{
selectie2 = 1;           

}

if (selectie3 < 1)
{
selectie2 = 0;

}

double aaa,bbb,theta,phi,offy,hx;
double radiusHK, radiusJH, radiusIJ, radiusRI, radiusVR, radiusBV, radiusUB;
double hxHK, hxJH, hxIJ, hxRI, hxVR, hxBV, hxUB;

double minHK, maxHK;
double minJH, maxJH;
double minIJ, maxIJ;
double minRI, maxRI;
double minVR, maxVR;
double minBV, maxBV;
double minUB, maxUB;


minHK=values[3][14];
maxHK=values[0][14];

minJH=values[3][12];
maxJH=values[0][12];

minIJ=values[3][10];
maxIJ=values[0][10];

minRI=values[3][8];
maxRI=values[0][8];

minVR=values[3][6];
maxVR=values[0][6];

minBV=values[3][4];
maxBV=values[0][4];

minUB=values[3][2];
maxUB=values[0][2];



offy=fabs((MAXLIMIT-MAG)/(MAXLIMIT-MINLIMIT));


double uu,bb,vv,rr,ii,jj,hh,kk;

if ((band=='U') || (band=='u'))
       {    
       uu=A_U-A_U;
       bb=A_B-A_U;
       vv=A_V-A_U;
       rr=A_R-A_U;
       ii=A_I-A_U;
       jj=A_J-A_U;
       hh=A_H-A_U;
       kk=A_K-A_U;
       }

if ((band=='B') || (band=='b'))
       {    
       uu=A_U-A_B;
       bb=A_B-A_B;
       vv=A_V-A_B;
       rr=A_R-A_B;
       ii=A_I-A_B;
       jj=A_J-A_B;
       hh=A_H-A_B;
       kk=A_K-A_B;
       }

if ((band=='V') || (band=='v'))
       {    
       uu=A_U-A_V;
       bb=A_B-A_V;
       vv=A_V-A_V;
       rr=A_R-A_V;
       ii=A_I-A_V;
       jj=A_J-A_V;
       hh=A_H-A_V;
       kk=A_K-A_V;
       }

if ((band=='R') || (band=='r'))
       {    
       uu=A_U-A_R;
       bb=A_B-A_R;
       vv=A_V-A_R;
       rr=A_R-A_R;
       ii=A_I-A_R;
       jj=A_J-A_R;
       hh=A_H-A_R;
       kk=A_K-A_R;
       }

if ((band=='I') || (band=='i'))
       {    
       uu=A_U-A_I;
       bb=A_B-A_I;
       vv=A_V-A_I;
       rr=A_R-A_I;
       ii=A_I-A_I;
       jj=A_J-A_I;
       hh=A_H-A_I;
       kk=A_K-A_I;
       }

if ((band=='J') || (band=='j'))
       {    
       uu=A_U-A_J;
       bb=A_B-A_J;
       vv=A_V-A_J;
       rr=A_R-A_J;
       ii=A_I-A_J;
       jj=A_J-A_J;
       hh=A_H-A_J;
       kk=A_K-A_J;
       }

if ((band=='H') || (band=='h'))
       {    
       uu=A_U-A_H;
       bb=A_B-A_H;
       vv=A_V-A_H;
       rr=A_R-A_H;
       ii=A_I-A_H;
       jj=A_J-A_H;
       hh=A_H-A_H;
       kk=A_K-A_H;
       }

if ((band=='K') || (band=='k'))
       {    
       uu=A_U-A_K;
       bb=A_B-A_K;
       vv=A_V-A_K;
       rr=A_R-A_K;
       ii=A_I-A_K;
       jj=A_J-A_K;
       hh=A_H-A_K;
       kk=A_K-A_K;
       }


hxHK=(MAG+kk-values[1][13])*(MAG+kk-values[2][13])*(MAG+kk-values[3][13]) * values[0][14] / (values[0][13]-values[1][13])/(values[0][13]-values[2][13])/(values[0][13]-values[3][13]) + (MAG+kk-values[0][13])*(MAG+kk-values[2][13])*(MAG+kk-values[3][13]) * values[1][14] / (values[1][13]-values[0][13])/(values[1][13]-values[2][13])/(values[1][13]-values[3][13]) + (MAG+kk-values[0][13])*(MAG+kk-values[1][13])*(MAG+kk-values[3][13]) * values[2][14] / (values[2][13]-values[0][13])/(values[2][13]-values[1][13])/(values[2][13]-values[3][13]) + (MAG+kk-values[0][13])*(MAG+kk-values[1][13])*(MAG+kk-values[2][13]) * values[3][14] / (values[3][13]-values[0][13])/(values[3][13]-values[1][13])/(values[3][13]-values[2][13]);



hxJH=(MAG+hh-values[1][11])*(MAG+hh-values[2][11])*(MAG+hh-values[3][11]) * values[0][12] / (values[0][11]-values[1][11])/(values[0][11]-values[2][11])/(values[0][11]-values[3][11]) + (MAG+hh-values[0][11])*(MAG+hh-values[2][11])*(MAG+hh-values[3][11]) * values[1][12] / (values[1][11]-values[0][11])/(values[1][11]-values[2][11])/(values[1][11]-values[3][11]) + (MAG+hh-values[0][11])*(MAG+hh-values[1][11])*(MAG+hh-values[3][11]) * values[2][12] / (values[2][11]-values[0][11])/(values[2][11]-values[1][11])/(values[2][11]-values[3][11]) + (MAG+hh-values[0][11])*(MAG+hh-values[1][11])*(MAG+hh-values[2][11]) * values[3][12] / (values[3][11]-values[0][11])/(values[3][11]-values[1][11])/(values[3][11]-values[2][11]);




hxIJ=(MAG+jj-values[1][9])*(MAG+jj-values[2][9])*(MAG+jj-values[3][9]) * values[0][10] / (values[0][9]-values[1][9])/(values[0][9]-values[2][9])/(values[0][9]-values[3][9]) + (MAG+jj-values[0][9])*(MAG+jj-values[2][9])*(MAG+jj-values[3][9]) * values[1][10] / (values[1][9]-values[0][9])/(values[1][9]-values[2][9])/(values[1][9]-values[3][9]) + (MAG+jj-values[0][9])*(MAG+jj-values[1][9])*(MAG+jj-values[3][9]) * values[2][10] / (values[2][9]-values[0][9])/(values[2][9]-values[1][9])/(values[2][9]-values[3][9]) + (MAG+jj-values[0][9])*(MAG+jj-values[1][9])*(MAG+jj-values[2][9]) * values[3][10] / (values[3][9]-values[0][9])/(values[3][9]-values[1][9])/(values[3][9]-values[2][9]);


hxRI=(MAG+ii-values[1][7])*(MAG+ii-values[2][7])*(MAG+ii-values[3][7]) * values[0][8] / (values[0][7]-values[1][7])/(values[0][7]-values[2][7])/(values[0][7]-values[3][7]) + (MAG+ii-values[0][7])*(MAG+ii-values[2][7])*(MAG+ii-values[3][7]) * values[1][8] / (values[1][7]-values[0][7])/(values[1][7]-values[2][7])/(values[1][7]-values[3][7]) + (MAG+ii-values[0][7])*(MAG+ii-values[1][7])*(MAG+ii-values[3][7]) * values[2][8] / (values[2][7]-values[0][7])/(values[2][7]-values[1][7])/(values[2][7]-values[3][7]) + (MAG+ii-values[0][7])*(MAG+ii-values[1][7])*(MAG+ii-values[2][7]) * values[3][8] / (values[3][7]-values[0][7])/(values[3][7]-values[1][7])/(values[3][7]-values[2][7]);


hxVR=(MAG+rr-values[1][5])*(MAG+rr-values[2][5])*(MAG+rr-values[3][5]) * values[0][6] / (values[0][5]-values[1][5])/(values[0][5]-values[2][5])/(values[0][5]-values[3][5]) + (MAG+rr-values[0][5])*(MAG+rr-values[2][5])*(MAG+rr-values[3][5]) * values[1][6] / (values[1][5]-values[0][5])/(values[1][5]-values[2][5])/(values[1][5]-values[3][5]) + (MAG+rr-values[0][5])*(MAG+rr-values[1][5])*(MAG+rr-values[3][5]) * values[2][6] / (values[2][5]-values[0][5])/(values[2][5]-values[1][5])/(values[2][5]-values[3][5]) + (MAG+rr-values[0][5])*(MAG+rr-values[1][5])*(MAG+rr-values[2][5]) * values[3][6] / (values[3][5]-values[0][5])/(values[3][5]-values[1][5])/(values[3][5]-values[2][5]);


hxBV=(MAG+vv-values[1][3])*(MAG+vv-values[2][3])*(MAG+vv-values[3][3]) * values[0][4] / (values[0][3]-values[1][3])/(values[0][3]-values[2][3])/(values[0][3]-values[3][3]) + (MAG+vv-values[0][3])*(MAG+vv-values[2][3])*(MAG+vv-values[3][3]) * values[1][4] / (values[1][3]-values[0][3])/(values[1][3]-values[2][3])/(values[1][3]-values[3][3]) + (MAG+vv-values[0][3])*(MAG+vv-values[1][3])*(MAG+vv-values[3][3]) * values[2][4] / (values[2][3]-values[0][3])/(values[2][3]-values[1][3])/(values[2][3]-values[3][3]) + (MAG+vv-values[0][3])*(MAG+vv-values[1][3])*(MAG+vv-values[2][3]) * values[3][4] / (values[3][3]-values[0][3])/(values[3][3]-values[1][3])/(values[3][3]-values[2][3]);

hxUB=(MAG+bb-values[1][1])*(MAG+bb-values[2][1])*(MAG+bb-values[3][1]) * values[0][2] / (values[0][1]-values[1][1])/(values[0][1]-values[2][1])/(values[0][1]-values[3][1]) + (MAG+bb-values[0][1])*(MAG+bb-values[2][1])*(MAG+bb-values[3][1]) * values[1][2] / (values[1][1]-values[0][1])/(values[1][1]-values[2][1])/(values[1][1]-values[3][1]) + (MAG+bb-values[0][1])*(MAG+bb-values[1][1])*(MAG+bb-values[3][1]) * values[2][2] / (values[2][1]-values[0][1])/(values[2][1]-values[1][1])/(values[2][1]-values[3][1]) + (MAG+bb-values[0][1])*(MAG+bb-values[1][1])*(MAG+bb-values[2][1]) * values[3][2] / (values[3][1]-values[0][1])/(values[3][1]-values[1][1])/(values[3][1]-values[2][1]);






phi=(rand()/(RAND_MAX+1.0))*PI;


double totrandom;
totrandom=(rand()/(RAND_MAX+1.0));



int splitlr;
splitlr = (rand())%6; // 0,1,2,3,4,5
int totceva;
totceva = 1+(rand())%2;

if (splitlr<=3)
{

radiusHK=2*fabs((maxHK-minHK))*exp(1.*offy)*(1.-offy)*(1.+offy)*(1.+offy)*totrandom*fabs(cos(phi));
radiusJH=2*fabs((maxJH-minJH))*exp(1.*offy)*(1.-offy)*(1.+offy)*(1.+offy)*totrandom*fabs(cos(phi));
radiusIJ=2*fabs((maxIJ-minIJ))*exp(1.*offy)*(1.-offy)*(1.+offy)*(1.+offy)*totrandom*fabs(cos(phi));
radiusRI=2*fabs((maxRI-minRI))*exp(1.*offy)*(1.-offy)*(1.+offy)*(1.+offy)*totrandom*fabs(cos(phi));
radiusVR=2*fabs((maxVR-minVR))*exp(1.*offy)*(1.-offy)*(1.+offy)*(1.+offy)*totrandom*fabs(cos(phi));
radiusBV=2*fabs((maxBV-minBV))*exp(1.*offy)*(1.-offy)*(1.+offy)*(1.+offy)*totrandom*fabs(cos(phi));
radiusUB=2*fabs((maxUB-minUB))*exp(1.*offy)*(1.-offy)*(1.+offy)*(1.+offy)*totrandom*fabs(cos(phi));

}

if (splitlr>3)
{

radiusHK=-4./3.*fabs((maxHK-minHK))*exp(3.*offy)*(1.-offy)*(offy)*(offy)*(offy)*totrandom*fabs(cos(phi));
radiusJH=-4./3.*fabs((maxJH-minJH))*exp(3.*offy)*(1.-offy)*(offy)*(offy)*(offy)*totrandom*fabs(cos(phi));
radiusIJ=-4./3.*fabs((maxIJ-minIJ))*exp(3.*offy)*(1.-offy)*(offy)*(offy)*(offy)*totrandom*fabs(cos(phi));
radiusRI=-4./3.*fabs((maxRI-minRI))*exp(3.*offy)*(1.-offy)*(offy)*(offy)*(offy)*totrandom*cos(phi);
radiusVR=-4./3.*fabs((maxVR-minVR))*exp(3.*offy)*(1.-offy)*(offy)*(offy)*(offy)*totrandom*cos(phi);
radiusBV=-4./3.*fabs((maxBV-minBV))*exp(3.*offy)*(1.-offy)*(offy)*(offy)*(offy)*totrandom*cos(phi);
radiusUB=-4./3.*fabs((maxUB-minUB))*exp(3.*offy)*(1.-offy)*(offy)*(offy)*(offy)*totrandom*cos(phi);

}


HK=hxHK+radiusHK;
JH=hxJH+radiusJH;
IJ=hxIJ+radiusIJ;
RI=hxRI+radiusRI;
VR=hxVR+radiusVR;
BV=hxBV+radiusBV;
UB=hxUB+radiusUB;


//// 


    if ((band=='U') || (band=='u'))
       {       
       DeltaMagU=0.;       
       DeltaMagB=-UB+A_B-A_U;
       DeltaMagV=-UB-BV+A_V-A_U;       
       DeltaMagR=-UB-BV-VR+A_R-A_U;
       DeltaMagI=-UB-BV-VR-RI+A_I-A_U;
       DeltaMagJ=-UB-BV-VR-RI-IJ+A_J-A_U;
       DeltaMagH=-UB-BV-VR-RI-IJ-JH+A_H-A_U;
       DeltaMagK=-UB-BV-VR-RI-IJ-JH-HK+A_K-A_U;
       }
       
    if ((band=='B') || (band=='b'))
       {       
       DeltaMagU=UB+A_U-A_B;       
       DeltaMagB=0.;
       DeltaMagV=-BV+A_V-A_B;       
       DeltaMagR=-BV-VR+A_R-A_B;
       DeltaMagI=-BV-VR-RI+A_I-A_B;
       DeltaMagJ=-BV-VR-RI-IJ+A_J-A_B;
       DeltaMagH=-BV-VR-RI-IJ-JH+A_H-A_B;
       DeltaMagK=-BV-VR-RI-IJ-JH-HK+A_K-A_B;
       }   
       
    if ((band=='V') || (band=='v'))
       {       
       DeltaMagU=BV+UB+A_U-A_V;       
       DeltaMagB=BV+A_B-A_V;
       DeltaMagV=0.;       
       DeltaMagR=-VR+A_R-A_V;
       DeltaMagI=-VR-RI+A_I-A_V;
       DeltaMagJ=-VR-RI-IJ+A_J-A_V;
       DeltaMagH=-VR-RI-IJ-JH+A_H-A_V;
       DeltaMagK=-VR-RI-IJ-JH-HK+A_K-A_V;
       }   
       
    if ((band=='R') || (band=='r'))
       {       
       DeltaMagU=VR+BV+UB+A_U-A_R;       
       DeltaMagB=VR+BV+A_B-A_R;
       DeltaMagV=VR+A_V-A_R;       
       DeltaMagR=0.;
       DeltaMagI=-RI+A_I-A_R;
       DeltaMagJ=-RI-IJ+A_J-A_R;
       DeltaMagH=-RI-IJ-JH+A_H-A_R;
       DeltaMagK=-RI-IJ-JH-HK+A_K-A_R;
       }   
          
       

    if ((band=='I') || (band=='i'))
       {       
       DeltaMagU=RI+VR+BV+UB+A_U-A_I;       
       DeltaMagB=RI+VR+BV+A_B-A_I;
       DeltaMagV=RI+VR+A_V-A_I;
       DeltaMagR=RI+A_R-A_I;
       DeltaMagI=0.;
       DeltaMagJ=-IJ+A_J-A_I;
       DeltaMagH=-IJ-JH+A_H-A_I;
       DeltaMagK=-IJ-JH-HK+A_K-A_I;
       }

    if ((band=='J') || (band=='j'))
       {       
       DeltaMagU=IJ+RI+VR+BV+UB+(A_U-A_J);
       DeltaMagB=IJ+RI+VR+BV+(A_B-A_J);
       DeltaMagV=IJ+RI+VR+(A_V-A_J);       
       DeltaMagR=IJ+RI+(A_R-A_J);
       DeltaMagI=IJ+(A_I-A_J);
       DeltaMagJ=0.;
       DeltaMagH=-JH+(A_H-A_J);
       DeltaMagK=-JH-HK+(A_K-A_J);
       }   
         
    if ((band=='H') || (band=='h'))
       {       
       DeltaMagU=JH+IJ+RI+VR+BV+UB+(A_U-A_H);       
       DeltaMagB=JH+IJ+RI+VR+BV+(A_B-A_H);
       DeltaMagV=JH+IJ+RI+VR+(A_V-A_H);       
       DeltaMagR=JH+IJ+RI+(A_R-A_H);
       DeltaMagI=JH+IJ+(A_I-A_H);
       DeltaMagJ=JH+(A_J-A_H);
       DeltaMagH=0.;
       DeltaMagK=-HK+(A_K-A_H);
       }   
       
    if ((band=='K') || (band=='k'))
       {       
       DeltaMagU=HK+JH+IJ+RI+VR+BV+UB+A_U-A_K;       
       DeltaMagB=HK+JH+IJ+RI+VR+BV+A_B-A_K;
       DeltaMagV=HK+JH+IJ+RI+VR+A_V-A_K;       
       DeltaMagR=HK+JH+IJ+RI+A_R-A_K;
       DeltaMagI=HK+JH+IJ+A_I-A_K;
       DeltaMagJ=HK+JH+A_J-A_K;
       DeltaMagH=HK+A_H-A_K;
       DeltaMagK=0.;
       }   
       



file1.setf(ios::fixed,ios::floatfield);  
file1.precision(6);

file2.setf(ios::fixed,ios::floatfield);  
file2.precision(6);

file3.setf(ios::fixed,ios::floatfield);  
file3.precision(6);

file4.setf(ios::fixed,ios::floatfield);  
file4.precision(6);

file5.setf(ios::fixed,ios::floatfield);  
file5.precision(6);

file6.setf(ios::fixed,ios::floatfield);  
file6.precision(6);

file7.setf(ios::fixed,ios::floatfield);  
file7.precision(6);

file8.setf(ios::fixed,ios::floatfield);  
file8.precision(6);


file1 <<"100  "<<setw(12) << X*Xmax;
file1 <<"  "<< setw(12) << Y*Ymax << "  "<< setw(12) << MAG+DeltaMagU  << "\n";

file2 <<"100  "<<setw(12) << X*Xmax;
file2 <<"  "<< setw(12) << Y*Ymax << "  "<< setw(12) << MAG+DeltaMagB << "\n";

file3 <<"100  "<<setw(12) << X*Xmax;
file3 <<"  "<< setw(12) << Y*Ymax << "  "<< setw(12) << MAG+DeltaMagV<< "\n";

file4 <<"100  "<<setw(12) << X*Xmax;
file4 <<"  "<< setw(12) << Y*Ymax << "  "<< setw(12) << MAG+DeltaMagR << "\n";

file5 <<"100  "<<setw(12) << X*XmaxIR;
file5 <<"  "<< setw(12) << Y*YmaxIR << "  "<< setw(12) << MAG+DeltaMagI << "\n";

file6 <<"100  "<<setw(12) << X*XmaxIR;
file6 <<"  "<< setw(12) << Y*YmaxIR << "  "<< setw(12) << MAG+DeltaMagJ << "\n";

file7 <<"100  "<<setw(12) << X*XmaxIR;
file7 <<"  "<< setw(12) << Y*YmaxIR << "  "<< setw(12) << MAG+DeltaMagH<< "\n";

file8 <<"100  "<<setw(12) << X*XmaxIR;
file8 <<"  "<< setw(12) << Y*YmaxIR << "  "<< setw(12) << MAG+DeltaMagK << "\n";

file9.setf(ios::fixed,ios::floatfield);  
file9.precision(6);


/*
file9<<setw(9) << "-99     ";
file9<<setw(9) << "-99     ";
file9<<setw(9) << "-99     ";
file9<<setw(9) << "-99     ";
file9<<setw(9) << "-99     ";
*/

file9<<setw(12)<< MAG+DeltaMagU <<"  ";
file9<<setw(12)<< MAG+DeltaMagB <<"  ";
file9<<setw(12)<< MAG+DeltaMagV <<"  ";
file9<<setw(12)<< MAG+DeltaMagR <<"  ";
file9<<setw(12)<< MAG+DeltaMagI <<"  ";
file9<<setw(12)<< MAG+DeltaMagJ <<"  ";
file9<<setw(12)<< MAG+DeltaMagH <<"  ";
file9<<setw(12)<< MAG+DeltaMagK <<"  ";

file9<<setw(12) << X*Xmax;
file9 <<"  "<< setw(12) << Y*Ymax;


file9<<'\n';




}




} // q loop


cout << "...done !"<<'\n';
cout << "\n";





cout <<"Writing file background.U.list ...";
cout <<"... done !" <<"\n";

cout <<"Writing file background.B.list ...";
cout <<"... done !" <<"\n";

cout <<"Writing file background.V.list ...";
cout <<"... done !" <<"\n";

cout <<"Writing file background.R.list ...";
cout <<"... done !" <<"\n";

cout <<"Writing file background.I.list ...";
cout <<"... done !" <<"\n";

cout <<"Writing file background.J.list ...";
cout <<"... done !" <<"\n";

cout <<"Writing file background.H.list ...";
cout <<"... done !" <<"\n";

cout <<"Writing file background.K.list ...";
cout <<"... done !" <<"\n";

cout <<"Writing file field.plot ...";
cout <<"... done !" <<"\n";

file1.close();
file2.close();
file3.close();
file4.close();
file5.close();
file6.close();
file7.close();
file8.close();
file9.close();

cout <<'\n';

cout <<"... all done !" <<"\n";

cout <<'\n';

return 0;
}

