////////////////////////////////////////////////////////////////////////////////
//
//
//   Author       : Bogdan Popescu
//   Organization : University of Cincinnati
//   Program      : goimf2
//   Version      : 2.014
//   Date         : Dec 10, 2014
//   Tested       : g++ 4.6.3 x64 (Ubuntu 12.04 LTS x64)
//                  g++ 4.8.2 x64 (Ubuntu 14.04 LTS x64)     
//                   
//                  
//
////////////////////////////////////////////////////////////////////////////////
//
//    MASSCLEAN (MASSive CLuster Evolution and ANalysis Package)
//    Copyright (C) 2007-2014 BOGDAN POPESCU and MARGARET HANSON
//
//    goimf2.cc is part of MASSCLEAN.
//
//    MASSCLEAN is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation.
//
//    MASSCLEAN is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with MASSCLEAN; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//    or go to : http://www.gnu.org/licenses/gpl.html
//
////////////////////////////////////////////////////////////////////////////////


#include <cstdlib>
#include <ctime>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <iomanip> 

#include <sys/time.h>  
#include <time.h>
#include <unistd.h>

using namespace std;


double N1k(double M, double alpha1, double m1)
{

return (fabs(pow((M/m1),(-alpha1))*M));

}


double N2k(double M, double alpha2, double m1)
{

return (fabs(pow((M/m1),(-alpha2))*M));

}


double N3k(double M, double alpha2, double alpha3, double m1, double m2)
{

return (fabs(pow((m2/m1),(-alpha2))*pow((M/m2),(-alpha3))*M));

}





double N3M(double M, double alpha2, double alpha3, double m1, double m2,double K) 
{
return (fabs(pow((m2/m1),(-alpha2))*pow((M/m2),(-alpha3))*K));

}

/////////////////////////////////
///+++++++++++++++++++ NOU

double N2M(double M, double alpha2, double m1, double K) 
{
return (fabs(pow((M/m1),(-alpha2))*K));

}

///+++++++++++++++++++ NOU
/////////////////////////////////


double NM1(double M, double alpha1, double m1)
{
return (fabs(pow((M/m1),(-alpha1))));
}

double NM2(double M, double alpha2, double m1)
{
return (fabs(pow((M/m1),(-alpha2))));
}


double NM3(double M, double alpha2, double alpha3, double m1, double m2)
{
return (fabs(pow((m2/m1),(-alpha2))*pow((M/m2),(-alpha3))));


}




int main()
{

  double Mtotal; 
  int VERBOSE;
  
  /// prototypes
  double N1k(double,double,double);
  double N2k(double,double,double);
  double N3k(double,double,double,double,double);
  double N3M(double,double,double,double,double,double);

  double N2M(double,double,double,double); ///++++ NOU
  
  double NM1(double,double,double);
  double NM2(double,double,double);
  double NM3(double,double,double,double,double);


  cout << "\n";


  cout <<"MASSCLEAN 2.014 (MASSive CLuster Evolution and ANalysis Package)"<<"\n";

  cout <<"Copyright (C) 2007-2014 BOGDAN POPESCU and MARGARET HANSON,"<<"\n";

  cout <<"under GNU General Public License. For details see 'COPYING'."<<"\n";

  cout <<"The program 'goIMF2' is part of MASSCLEAN 2.014 CC (Clever Crocodile)."<<"\n";

  cout << "\n";
  cout <<"http://massclean.org/"<< '\n';
  cout <<"http://bogdan.massclean.org/"<< '\n';

  cout << "\n";


  cout <<"Popescu & Hanson 2009, AJ, 138, 1724"<< '\n';
  cout <<"Popescu & Hanson 2010, ApJL, 713, L21"<< '\n';
  cout <<"Popescu & Hanson 2010, ApJ, 724, 296"<< '\n';
  cout <<"Popescu, Hanson & Elmegreen 2012, ApJ, 751, 122"<< '\n';
  cout <<"Popescu & Hanson 2014, ApJ, 780, 27"<< '\n';


  cout << "\n";
  cout << "\n";

  
//  sleep(1);


//timeval timv;

//gettimeofday(&timv, NULL);

//cout << timv.tv_sec <<"  "<<timv.tv_usec<<" "<<timv.tv_sec*timv.tv_sec<<"\n";
    
  
//  srand((unsigned)time(0));

////////////////////////////////////////////////////////
timeval timv;
gettimeofday(&timv, NULL);

int randomini;
randomini=timv.tv_usec;
srand(randomini);
///////////////////////////////////////////////////////

//cout << rand()/(RAND_MAX+1.0)<<'\n';
//cout << rand()/(RAND_MAX+1.0)<<'\n';
  

  //srand((unsigned)time(0)*time(0));
  //srand(time(0));

  //int RANDOMNO=(rand()*1483);//%RAND_MAX;// /(RAND_MAX+1.0);
  //srand(RANDOMNO);
 
  chdir("ini.files");

  ifstream data0("cluster.ini");

  if (data0.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;
  }

  while (data0.peek()=='#') { data0.ignore(300,'\n');}

  data0 >> VERBOSE; data0.ignore(300,'\n');
  data0 >> Mtotal; data0.ignore(300,'\n');
  data0.ignore(300,'\n');

  data0.close();



  int headonoff;



  //headonoff=0;
  //headonoff=1;


  ifstream data13("header.ini");

  if (data13.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;
  }

  while (data13.peek()=='#') { data13.ignore(300,'\n');}

  data13 >> headonoff; 
  data13.ignore(300,'\n');

  data13.close();


////////////////////////////
/// 


if (VERBOSE==1)
{
  cout <<"Reading file cluster.ini ...";
  cout <<"... done !" <<"\n";
}
  
  cout << "The mass of the cluster is: ";


  cout <<"M_total =" <<Mtotal<<" M_Sun";

  cout << "\n";
  
  
  int NOBINS=0;
  
  double m0,m1,m2,m3,mmin,mmax,mtrunc,m4;
  double alpha1,alpha2,alpha3;
  double C1,C2;
  double h1,h2,h3,tol;
  double hn3,hn3read;
  double m2read;

  int noiter;

  int FLUCTUATIONS;

  
  int N1bins,N2bins,N3bins;
  int MIMFStype;
  

  
  cout.setf(ios::fixed,ios::floatfield);
  
if (VERBOSE==1)
{
  cout <<"\n";
  cout <<"Reading file imf.ini ...";
}
  
  ifstream data("imf.ini");

  if (data.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;
  }

  while (data.peek()=='#') { data.ignore(300,'\n');}

  data >> m0; data.ignore(300,'\n');
  data >> m1; data.ignore(300,'\n');
  data >> m2read; data.ignore(300,'\n');
  data >> m3; data.ignore(300,'\n');
  data >> mmin; data.ignore(300,'\n');
  data >> mmax; data.ignore(300,'\n');
  data >> mtrunc; data.ignore(300,'\n');
  data >> alpha1; data.ignore(300,'\n');
  data >> alpha2; data.ignore(300,'\n');
  data >> alpha3; data.ignore(300,'\n');

  data >> MIMFStype; data.ignore(300,'\n');


/*
  data >> N1bins; data.ignore(300,'\n');
  data >> N2bins; data.ignore(300,'\n');
  data >> N3bins; data.ignore(300,'\n');
  data >> hn3read; data.ignore(300,'\n');
  data >> noiter; data.ignore(300,'\n');
  data >> tol; data.ignore(300,'\n');
  
  data >> FLUCTUATIONS; 

  data.ignore(300,'\n');

*/

///////////////////////////////////////////////////////////// ^
/// 

  data.ignore(300,'\n');

  data.close();

  hn3=hn3read;
  m2=m2read;
//  if (Mtotal<mmax) {mmax=Mtotal;}

  chdir("..");

  
  /// -------------- nou

cout.precision(2);
cout <<"Kroupa IMF: alpha1="<<alpha1<<", alpha2="<<alpha2<<", alpha3="<<alpha3<<'\n'<<'\n';


//////////////////////////////////////////////////////////////////////////////////
/// 

double Im0m1;

Im0m1 = pow(m1,alpha1)/(2.-alpha1)*(pow(m1,2.-alpha1)-pow(m0,2.-alpha1));

//cout <<"Im0m1 "<<Im0m1<<'\n';
  

double Im1m2;

Im1m2 = pow(m1,alpha2)/(2.-alpha2)*(pow(m2,2.-alpha2)-pow(m1,2.-alpha2));

//cout <<"Im1m2 "<<Im1m2<<'\n';
  

double Im2infinit;

Im2infinit = pow((m2/m1),-alpha2)*pow(m2,alpha3)/(alpha3-2.)*pow(m2,2.-alpha3);

//cout <<"Im2infinit "<<Im2infinit<<'\n';
  
double Im2mtrunc;

if (mtrunc!=-99) 

{Im2mtrunc = pow((m2/m1),-alpha2)*pow(m2,alpha3)/(alpha3-2.)*(pow(m2,2.-alpha3)-pow(mtrunc,2.-alpha3));

//cout <<"Im2mtrunc "<<Im2mtrunc<<'\n';
  }


double Im2Mtotal;

Im2Mtotal = pow((m2/m1),-alpha2)*pow(m2,alpha3)/(alpha3-2.)*(pow(m2,2.-alpha3)-pow(Mtotal,2.-alpha3));

//cout <<"Im2Mtotal "<<Im2Mtotal<<'\n';
  

double K;

//K=Mtotal/(Im0m1+Im1m2+Im2infinit);

K=Mtotal/(Im0m1+Im1m2+Im2Mtotal);

if ((mtrunc!=-99) && (Mtotal>mtrunc)) {K=Mtotal/(Im0m1+Im1m2+Im2mtrunc);}

//cout <<"K "<<K<<'\n';



/// Analitic percentage

double M1, M21, M22, M31, M32;

M1=K*pow(m1,alpha1)/(alpha1-2.)*(pow(m0,2.-alpha1)-pow(m1,2.-alpha1));
M21=K*pow(m1,alpha2)/(alpha2-2.)*(pow(m1,2.-alpha2)-pow(mmin,2.-alpha2));
M22=K*pow(m1,alpha2)/(alpha2-2.)*(pow(mmin,2.-alpha2)-pow(m2,2.-alpha2));
M31=K*pow(m1/m2,alpha2)*pow(m2,alpha3)/(alpha3-2.)*(pow(m2,2.-alpha3)-pow(mmax,2.-alpha3));

if (Mtotal<mmax)
{
M31=K*pow(m1/m2,alpha2)*pow(m2,alpha3)/(alpha3-2.)*(pow(m2,2.-alpha3)-pow(Mtotal,2.-alpha3));
}

if (mtrunc>=0)
{
M32=K*pow(m1/m2,alpha2)*pow(m2,alpha3)/(alpha3-2.)*(pow(mmax,2.-alpha3)-pow(mtrunc,2.-alpha3));
}

if ((mtrunc<0) && (Mtotal>mmax))
{
M32=K*pow(m1/m2,alpha2)*pow(m2,alpha3)/(alpha3-2.)*(pow(mmax,2.-alpha3)-pow(Mtotal,2.-alpha3));
}

if ((mtrunc<0) && (Mtotal<mmax))
{
M32=0.;
}



//////////////




chdir("data");

ofstream outfile("n_distribution.out");

  if (outfile.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;

  }





  outfile.setf(ios::fixed,ios::floatfield); 

if (headonoff==1)
{
  outfile <<"##  "<< '\n';
  outfile <<"##  MASSCLEAN 2.014 Clever Crocodile"<< '\n';
  outfile <<"##  B.Popescu and M.M.Hanson, 2007-2014 "<< '\n';
  outfile <<"##  http://massclean.org/"<< '\n';

  outfile <<"##  "<< '\n';

  outfile <<"##  Popescu & Hanson 2009, AJ, 138, 1724"<< '\n';
  outfile <<"##  Popescu & Hanson 2010, ApJL, 713, L21"<< '\n';
  outfile <<"##  Popescu & Hanson 2010, ApJ, 724, 296"<< '\n';
  outfile <<"##  Popescu, Hanson & Elmegreen 2012, ApJ, 751, 122"<< '\n';
  outfile <<"##  Popescu & Hanson 2014, ApJ, 780, 27"<< '\n';

  outfile <<"##  "<< '\n';
  
  outfile <<"#   n_distribution.out"<< '\n';
  outfile <<"#"<< '\n';

  outfile.precision(2);
  outfile <<"#   The total mass : "<<Mtotal<<" M_Sun"<< '\n';
 

  
  outfile <<"#"<< '\n';
}

  
  outfile.precision(2);
/*  
  outfile<<"# Stellar populations :"<<"\n";
  outfile<<"# "<< setw(10)<<"("<<m0<<","<<m1<<") "<< setw(4)<<"M_Sun";
 
  outfile<< setw(10)<<I1fin/(I1fin+I2fin+I3FINAL)*100.<<"%"<< setw(4) <<"= ";
 
  outfile<< setw(8) <<I1fin/(I1fin+I2fin+I3FINAL)*Mtotal<< setw(6)<<" M_Sun"<<"\n";
  
 
  
   outfile<<"# "<<setw(10)<<"("<<m1<<","<<m2<<") "<< setw(6)<<"M_Sun";

   outfile<<setw(10)<<(I2fin)/(I1fin+I2fin+I3FINAL)*100.<<"%"<< setw(4)<< "= ";

   outfile<<setw(7)<<(I2fin)/(I1fin+I2fin+I3FINAL)*Mtotal<< setw(6)<<" M_Sun"<<"\n";
  

  
  outfile<<"# "<<setw(10)<<"("<<m2<<","<<mmax<<") "<< setw(7)<<"M_Sun";

   outfile<< setw(10)<<(I3fin)/(I1fin+I2fin+I3FINAL)*100.<<"%"<< setw(4) <<"= ";

   outfile<< setw(7)<<(I3fin)/(I1fin+I2fin+I3FINAL)*Mtotal<< setw(6)<<" M_Sun"<<"\n";
  
*/
/////


outfile.precision(10);


/////////////

double Mini, Mfin;
double Mini2, Mfin2;
double MOUT, MOUT2;
MOUT2=0;

//double mramas=(M22+M31-MOUT2);

int wbin;
wbin=(int)(log10(Mtotal))+1;

int i;

int MARE=0;

Mini2=mmin;

//for (i=0;i<10;i++)
while (Mfin2<=m2)
{
Mfin2=pow( pow(Mini2,(-alpha2+1.)) -(alpha2-1.)/K*pow(m1,-alpha2)  ,1./(-alpha2+1.));

if (Mfin2<=m2)
{
MOUT=Mini2+(Mfin2-Mini2)*rand()/(RAND_MAX+1.0);
MOUT2=MOUT2+MOUT;


NOBINS=NOBINS+1;

cout.precision(10);
//cout <<"Bin no. "<<NOBINS<<"  "<<MOUT<<"    1  "<<Mini2<<"    "<<Mfin2<<" ramas "<<(M22+M31-MOUT2)<<'\n';

////cout <<"Bin no. "<<NOBINS<<"  "<<MOUT<<"    1  "<<Mini2<<"    "<<Mfin2<<'\n';



outfile<< setw(14)  <<MOUT<<" "<< setw(8)<<"1"<<"   "<<Mini2<<"   "<<Mfin2 <<'\n'; 

////////////////////////////////////////////////////////////////////////////////////////////////
/// poate 10k intr-un fisier

if ((Mtotal<=100.) || (NOBINS==1))
{
cout <<"Bin no. "<<setw(wbin)<< NOBINS <<" ... done !"<<'\n';

if (Mtotal>=1000000.) {cout <<" ... please wait ... "<<'\n';}

if (Mtotal>100.) {cout <<" (displaying only every "<<(int)(pow(10.,(int)(log10(Mtotal))))<<"th bin) "<<'\n';}

}

if ((Mtotal>100.) && (NOBINS%(int)(pow(10.,(int)(log10(Mtotal))))==0 ))
{
cout <<"Bin no. "<<setw(wbin)<< NOBINS <<" ... done !"<<'\n';
}

///  1k-10k :1000
/// 10k-100k:10000
//// NOBINS%(int)(pow(10.,(int)(log10(Mtotal))))


Mini2=Mfin2;
}

//Mini2=Mfin2;


}



//cout <<"Mfin2 "<<Mini2<<'\n';

if (Mini2<m2)
{
double Nrest;
Nrest=K*pow(m1,alpha2)/(alpha2-1)*(pow(Mini2,-alpha2+1)-pow(m2,-alpha2+1));
//cout <<"Nrest "<<Nrest<<'\n';

Mfin=pow( pow(m2,(-alpha3+1.)) -(1.-Nrest)*(alpha3-1.)/K*pow((m2/m1),alpha2)*pow(m2,-alpha3)  ,1./(-alpha3+1.));
//cout <<"Mfin "<<Mfin<<'\n';

/////////////
double N3,N3rest;

N3=K/(alpha3-1.)*pow((m1/m2),alpha2)*pow(m2,alpha3)*(pow(Mfin,-alpha3+1.)-pow(Mtotal,-alpha3+1.));
N3rest=fmod(N3,1.);
//cout << N3rest<<"  "<< N3 <<" "<<(int)(N3) <<'\n';

double Mfinint;

//Mfinint=pow( pow(Mfin,-alpha3+1.)-(1.-Nrest)/K*(alpha3-1.)*pow((m2/m1),alpha2)*pow(m2,-alpha3)  ,1./(-alpha3+1.));

Mfinint=pow( pow(Mtotal,-alpha3+1.)+((int)(N3))/K*(alpha3-1.)*pow((m2/m1),alpha2)*pow(m2,-alpha3)  ,1./(-alpha3+1.));


//cout << Mfinint <<'\n';

////////////

MOUT=Mini2+(Mfinint-Mini2)*rand()/(RAND_MAX+1.0);
MOUT2=MOUT2+MOUT;
NOBINS=NOBINS+1;

////cout <<"Bin no. "<<NOBINS<<"  "<<MOUT<<"    1  "<<Mini2<<"    "<<Mfin<<" ramas "<<(M22+M31-MOUT2)<<'\n';

int OUTbin=0; ////?????

if (OUTbin==1)
{
outfile <<"# " << setw(14) <<MOUT<<" "<< setw(8)<<"1"<<"   "<<Mini2<<"   "<<Mfinint <<'\n'; 
cout <<"Bin no. "<<setw(wbin)<< NOBINS <<" ... done !"<<'\n';
}

if (OUTbin==0)
{
outfile<< setw(14)  <<MOUT<<" "<< setw(8)<<"1"<<"   "<<Mini2<<"   "<<Mfinint <<'\n'; 


if (Mtotal<=100.)
{
cout <<"Bin no. "<<setw(wbin)<< NOBINS <<" ... done !"<<'\n';
}

if ((Mtotal>100.) && (NOBINS%(int)(pow(10.,(int)(log10(Mtotal))))==0 ))
{
cout <<"Bin no. "<<setw(wbin)<< NOBINS <<" ... done !"<<'\n';
}


}


//Mini=Mfin;
Mini=Mfinint;
}


if (Mini2==m2)
{
Mini=m2;
}

//for (i=0;i<100;i++)


double Mfinout;

//while (Mfin<=mmax)
while (Mfin<=Mtotal)
{


Mfinout=Mini;

//double mramas=(M22+M31-MOUT2);

double mramas=Mtotal-MOUT2-M1-M21;
//ramasmare=Mtotal-MOUT2-M1-M21;

double N3limit, N3infinit, N3Mtotal;

N3infinit=K/(alpha3-1.)*pow((m1/m2),alpha2)*pow(m2,alpha3)*pow(Mini,-alpha3+1.);

if (Mtotal<=mmax)
{
N3Mtotal=N3infinit-K/(alpha3-1.)*pow((m1/m2),alpha2)*pow(m2,alpha3)*pow(Mtotal,-alpha3+1.);
//N3limit=N3Mtotal;
N3limit=(int)(N3Mtotal);
}

if (Mtotal>mmax)
{
//N3limit=N3infinit;
N3Mtotal=N3infinit-K/(alpha3-1.)*pow((m1/m2),alpha2)*pow(m2,alpha3)*pow(Mtotal,-alpha3+1.);
N3limit=(int)(N3Mtotal);
}


if (mramas>0)
{
Mfin=pow( pow(Mini,(-alpha3+1.)) -(alpha3-1.)/K*pow((m2/m1),alpha2)*pow(m2,-alpha3)  ,1./(-alpha3+1.));

if (N3limit>0.) //1.
{
MOUT=Mini+(Mfin-Mini)*rand()/(RAND_MAX+1.0);

if ((M22+M31-MOUT2-MOUT)>0)
{

if ((MOUT>=mmin) && (MOUT<=mmax)) /// 
{
MOUT2=MOUT2+MOUT;
NOBINS=NOBINS+1;
////cout <<"Bin no. "<<NOBINS<<"  "<<MOUT<<"    1  "<<Mini<<"    "<<Mfin<<" "<<M22+M31-MOUT2<<'\n';

outfile<< setw(14)  <<MOUT<<" "<< setw(8)<<"1"<<"   "<<Mini<<"   "<<Mfin <<'\n'; 


if (Mtotal<=100.)
{
cout <<"Bin no. "<<setw(wbin)<< NOBINS <<" ... done !"<<'\n';
}

if ((Mtotal>100.) && (NOBINS%(int)(pow(10.,(int)(log10(Mtotal))))==0 ))
{
cout <<"Bin no. "<<setw(wbin)<< NOBINS <<" ... done !"<<'\n';
}


}

///cout.precision(2);
///cout <<"N "<<N3limit<< '\n';

//cout.precision(2);
//cout <<"Mr "<< Mtotal-MOUT2-M1-M21 <<" N "<<N3limit<<'\n';

}

}

//cout <<"N3limit : "<<N3limit<<'\n';




if (N3limit<=-1.)  //1
{

//if (mramas>Mfin)
if (mramas>Mini)
{
MOUT=Mini+(mramas-Mini)*rand()/(RAND_MAX+1.0);

//if ((M22+M31-MOUT2-MOUT)>0)
if ((MOUT>=mmin) && (MOUT<=mmax)) ///
{
MOUT2=MOUT2+MOUT;
NOBINS=NOBINS+1;
////cout <<"Bin no. "<<NOBINS<<"  "<<MOUT<<"    1x  "<<Mini<<"    "<<mramas<<" "<<M22+M31-MOUT2<<'\n';

outfile<< setw(14)  <<MOUT<<" "<< setw(8)<<"1x1"<<"   "<<Mini<<"   "<<mramas <<'\n'; ////????
cout <<"Bin no. "<<setw(wbin)<< NOBINS <<"1 ... done !"<<'\n'; ////????? x1

///cout.precision(2);
///cout <<"Last 1? "<< Mtotal-MOUT2-M1-M21 <<" N "<<N3limit<<" Mini "<<Mini<<" Mfin "<<Mfin<<'\n';

}

}


break;
}///





/*
if (N3limit<2)
{
MOUT=Mini+((mramas)-Mini)*rand()/(RAND_MAX+1.0);

if ((M22+M31-MOUT2)>0)
{
MOUT2=MOUT2+MOUT;
NOBINS=NOBINS+1;
cout <<"Bin no. "<<NOBINS<<"  "<<MOUT<<"    1x  "<<Mini<<"    "<<(mramas)<<" "<<N3limit<<" ramas fin "<<(M22+M31-MOUT2)<<'\n';
break;
}


}
*/




/*
if ((M22+M31-MOUT2-MOUT)>0)
{
MOUT2=MOUT2+MOUT;
NOBINS=NOBINS+1;
cout <<"Bin no. "<<NOBINS<<"  "<<MOUT<<"    1  "<<Mini<<"    "<<Mfin<<" "<<N3limit<<'\n';
}
*/


}

Mini=Mfin;
}

//cout <<"out "<<MOUT2<<" ramas "<<(M22+M31-MOUT2)<<'\n';

//cout <<"N3lim "<<N3limit<<" ramas "<<(M22+M31-MOUT2)<<'\n';

////cout <<"MARE "<<MARE<<" ramas adica "<<(M22+M31+M32-MOUT2)<<'\n';

//double N3limlast;
//N3limlast=K/(alpha3-1.)*pow((m1/m2),alpha2)*pow(m2,alpha3)*pow(Mini,-alpha3+1.);




double aduna;
//if (M32>mmax) { aduna=0.;}
if (M32>mmax) { aduna=M32;}
if (M32<=mmax) { aduna=M32;}

///cout <<"Mfin "<<Mfinout<<" ramas "<<(M22+M31-MOUT2+aduna)<<'\n';

///////////////////////////////////////////
///// +++++ ramas > mmax

if ((M22+M31+M32-MOUT2)>mmax)
{

while ((M22+M31+M32-MOUT2)>mmax)
{
MOUT=m2+(mmax-m2)*rand()/(RAND_MAX+1.0);
MOUT2=MOUT2+MOUT;
NOBINS=NOBINS+1;

outfile<< setw(14)  <<MOUT<<" "<< setw(8)<<"1"<<"   "<<m2<<"   "<<mmax <<'\n'; 

cout <<"Bin no. "<<setw(wbin)<< NOBINS <<" ... done !"<<'\n';

////cout <<"over mmax "<< MOUT<<" "<<'\n';

}//while

}
///// +++++ ramas > mmax
//////////////////////////////////////////


int randbin3;
//randbin3=rand()%2;

if ((MIMFStype==1))
{
randbin3=rand()%3;
}

if ((MIMFStype==2))
//if ((MIMFStype==2))
{
randbin3=5;
}



//////////////////////////------>>>>>>>>>>>>>>>
//
//randbin3=5;
//
//////////////////////////------>>>>>>>>>>>>>>>

if ((M22+M31-MOUT2+aduna>Mfinout) && (randbin3==1))
{



MOUT=Mfinout+((M22+M31-MOUT2+aduna)-Mfinout)*rand()/(RAND_MAX+1.0);

if ((MOUT>=mmin) && (MOUT<=mmax)) ///
{
MOUT2=MOUT2+MOUT;
NOBINS=NOBINS+1;

////cout <<"Bin no. "<<NOBINS<<"  "<<MOUT<<"    1xx  "<<(M22+M31-MOUT2+MOUT)-Mtotal/10000.<<"    "<<(M22+M31-MOUT2+MOUT)+Mtotal/10000.<<'\n';

outfile<< setw(14)  <<MOUT<<" "<< setw(8)<<"1"<<"   "<<Mfinout<<"   "<<(M22+M31-MOUT2+MOUT+aduna) <<'\n'; 
cout <<"Bin no. "<<setw(wbin)<< NOBINS <<" ... done !"<<'\n';

///cout <<"3 iteratii "<< MOUT<<" "<<aduna<<'\n';
}

}


////////////////////////////

if ((M22+M31-MOUT2+aduna)>0. && (randbin3==0))
{



MOUT=m2+((M22+M31-MOUT2+aduna)-m2)*rand()/(RAND_MAX+1.0);

if ((MOUT>=mmin) && (MOUT<=mmax)) ///
{
MOUT2=MOUT2+MOUT;
NOBINS=NOBINS+1;

////cout <<"Bin no. "<<NOBINS<<"  "<<MOUT<<"    1xx  "<<(M22+M31-MOUT2+MOUT)-Mtotal/10000.<<"    "<<(M22+M31-MOUT2+MOUT)+Mtotal/10000.<<'\n';

outfile<< setw(14)  <<MOUT<<" "<< setw(8)<<"1"<<"   "<<m2<<"   "<<(M22+M31-MOUT2+MOUT+aduna) <<'\n'; 
cout <<"Bin no. "<<setw(wbin)<< NOBINS <<" ... done !"<<'\n';

///cout <<"2 iteratii "<< MOUT<<" "<<aduna<<'\n';
}

}

/////////////////////////////





////////////////////////////


int randbin44;

if ((MIMFStype==1))
{
randbin44=rand()%2;
}

//cout <<"MIMFStype "<< MIMFStype<<'\n';
//////////////////////////------>>>>>>>>>>>>>>>
//
//if ((MIMFStype==2) || (MIMFStype==3))
if ((MIMFStype==2))
{
randbin44=5;
}
//
//////////////////////////------>>>>>>>>>>>>>>>


if ((M22+M31-MOUT2+aduna)>0. && (randbin44==0))
{



MOUT=m2+((M22+M31-MOUT2+aduna)-m2)*rand()/(RAND_MAX+1.0);

if ((MOUT>=mmin) && (MOUT<=mmax)) ///
{
MOUT2=MOUT2+MOUT;
NOBINS=NOBINS+1;

////cout <<"Bin no. "<<NOBINS<<"  "<<MOUT<<"    1xx  "<<(M22+M31-MOUT2+MOUT)-Mtotal/10000.<<"    "<<(M22+M31-MOUT2+MOUT)+Mtotal/10000.<<'\n';

outfile<< setw(14)  <<MOUT<<" "<< setw(8)<<"1"<<"   "<<m2<<"   "<<(M22+M31-MOUT2+MOUT+aduna) <<'\n'; 
cout <<"Bin no. "<<setw(wbin)<< NOBINS <<" ... done !"<<'\n';

///cout <<"4.1 iteratii "<< MOUT<<" "<<(M22+M31-MOUT2+MOUT+aduna)<<'\n';
}


if ((M22+M31-MOUT2+aduna)>0.)
{

MOUT=m2+((M22+M31-MOUT2+aduna)-m2)*rand()/(RAND_MAX+1.0);

if ((MOUT>=mmin) && (MOUT<=mmax)) ///
{
MOUT2=MOUT2+MOUT;
NOBINS=NOBINS+1;

////cout <<"Bin no. "<<NOBINS<<"  "<<MOUT<<"    1xx  "<<(M22+M31-MOUT2+MOUT)-Mtotal/10000.<<"    "<<(M22+M31-MOUT2+MOUT)+Mtotal/10000.<<'\n';

outfile<< setw(14)  <<MOUT<<" "<< setw(8)<<"1"<<"   "<<m2<<"   "<<(M22+M31-MOUT2+MOUT+aduna) <<'\n'; 
cout <<"Bin no. "<<setw(wbin)<< NOBINS <<" ... done !"<<'\n';

///cout <<"4.2 iteratii "<< MOUT<<" "<<(M22+M31-MOUT2+MOUT+aduna)<<'\n';
}
}


if ((M22+M31-MOUT2+aduna)>0.)
{

MOUT=m2+((M22+M31-MOUT2+aduna)-m2)*rand()/(RAND_MAX+1.0);

if ((MOUT>=mmin) && (MOUT<=mmax)) ///
{
MOUT2=MOUT2+MOUT;
NOBINS=NOBINS+1;

////cout <<"Bin no. "<<NOBINS<<"  "<<MOUT<<"    1xx  "<<(M22+M31-MOUT2+MOUT)-Mtotal/10000.<<"    "<<(M22+M31-MOUT2+MOUT)+Mtotal/10000.<<'\n';

outfile<< setw(14)  <<MOUT<<" "<< setw(8)<<"1"<<"   "<<m2<<"   "<<(M22+M31-MOUT2+MOUT+aduna) <<'\n'; 
cout <<"Bin no. "<<setw(wbin)<< NOBINS <<" ... done !"<<'\n';

///cout <<"4.3 iteratii "<< MOUT<<" "<<(M22+M31-MOUT2+MOUT+aduna)<<'\n';
}
}


}

/////////////////////////////




if ((M22+M31-MOUT2+aduna)>0.)
{

int randbin;

randbin=rand()%2;

MOUT=(M22+M31-MOUT2+aduna)+pow(-1.,randbin)*Mtotal/10000.*rand()/(RAND_MAX+1.0);

if ((MOUT>=mmin) && (MOUT<=mmax)) ///
{
MOUT2=MOUT2+MOUT;
NOBINS=NOBINS+1;

////cout <<"Bin no. "<<NOBINS<<"  "<<MOUT<<"    1xx  "<<(M22+M31-MOUT2+MOUT)-Mtotal/10000.<<"    "<<(M22+M31-MOUT2+MOUT)+Mtotal/10000.<<'\n';

outfile<< setw(14)  <<MOUT<<" "<< setw(8)<<"1"<<"   "<<(M22+M31-MOUT2+MOUT+aduna)-Mtotal/10000.<<"   "<<(M22+M31-MOUT2+MOUT+aduna)+Mtotal/10000. <<'\n'; 
cout <<"Bin no. "<<setw(wbin)<< NOBINS <<" ... done !"<<'\n';
}

}



   cout.precision(2);
   
   
 


outfile.close();

  

cout<<"\n";

cout << "The mass of the cluster is: ";


  cout <<"M_total =" <<Mtotal<<" M_Sun";

  cout << "\n";


  
  cout.precision(2);
/*  
  cout<<"Stellar populations :"<<"\n";
  cout<< setw(10)<<"("<<m0<<","<<m1<<") "<< setw(8)<<"M_Sun";
  cout.precision(2);
  cout<< setw(10)<<I1fin/(I1fin+I2fin+I3FINAL)*100.<<"%"<< setw(4) <<"= ";
  cout.precision(2);
  cout<< setw(8) <<I1fin/(I1fin+I2fin+I3FINAL)*Mtotal<< setw(6)<<" M_Sun"<<"\n";
  
  cout.precision(2);
  
  cout<< setw(10)<<"("<<m1<<","<<m2<<") "<< setw(8)<<"M_Sun";
   cout.precision(2);
   cout<<setw(10)<<I2fin/(I1fin+I2fin+I3FINAL)*100.<<"%"<< setw(4)<< "= ";
   cout.precision(2);
   cout<<setw(8)<<I2fin/(I1fin+I2fin+I3FINAL)*Mtotal<< setw(6)<<" M_Sun"<<"\n";
  
  cout.precision(2);
  
  cout<< setw(10)<<"("<<m2<<","<<mmax<<") "<< setw(6)<<"M_Sun";
   cout.precision(2);
   cout<< setw(10)<<I3fin/(I1fin+I2fin+I3FINAL)*100.<<"%"<< setw(4) <<"= ";
   cout.precision(2);
   cout<< setw(8)<<I3fin/(I1fin+I2fin+I3FINAL)*Mtotal<< setw(6)<<" M_Sun"<<"\n";
  
  
  
  
  cout<<"Number of bins : "<<NOBINS<<"\n";
 
  
  
 cout <<"\n";
 cout <<"... done !" <<"\n";
 cout <<"\n";
*/ 
 
/* 
  cout <<M1<<" "<<M21<<"  "<<M22<<"  "<<M31<<"  "<<M32<<"\n";
  cout <<M22+M31+aduna<<"\n";
*/


  cout <<"Percentage (above "<< mmin <<" M_Sun): "<<((M1+M21+M32-aduna)+MOUT2)/Mtotal*100.<<"%"<<"\n";





cout <<"Mass included in the distribution: "<<MOUT2<<" M_Sun"<<"\n";  
cout <<"Number of stars: "<<NOBINS<<"\n"; 






 //chdir("..");
 
 //system("pwd");
 


//////////////////////////////

// chdir("data");
//
//ofstream outfile2("mass.out");
//
//  if (outfile2.bad())
//  {
//  cerr << "Error ! "<<'\n';
//  return 1 ;
//
//  }
  
 // outfile2 << Mtotal-I33final*K <<"   "<< Mtotal-I33final*K+MOUT2<<"   "<<(Mtotal-I33final*K+MOUT2)/Mtotal*100. ;
  
//  outfile2.close();
//  
//  chdir("..");

//cout.precision(10);
//cout << rand()/(RAND_MAX+1.0)<<'\n';
//cout << rand()/(RAND_MAX+1.0)<<'\n';
  
cout <<"\n";  
   cout <<"... all done !" <<"\n";

 cout <<"\n";



 return 0;
}
